import numpy as np
import matplotlib.pyplot as plt
import os

def twos_complement(value, bit_width):
    if value >= 2**(bit_width - 1):
        return value - 2**bit_width
    return value

def parse_hex_file(file_path):
    i_values = []
    q_values = []
    with open(file_path, 'r') as file:
        for line in file:
            line = line.strip()  # Remove any surrounding whitespace
            try:
                # Attempt to convert the line from hex to integer
                num = int(line, 16)
            except ValueError:
                # Skip the line if it cannot be converted to an integer
                continue

            # Extract I and Q values
            i = (num >> 12) & 0xFFF  # Top 12 bits
            q = num & 0xFFF  # Bottom 12 bits

            # Convert from unsigned to signed
            i_signed = twos_complement(i, 12)
            q_signed = twos_complement(q, 12)

            # Append to lists
            i_values.append(i_signed)
            q_values.append(q_signed)

    return i_values, q_values


def compute_amplitude(i_values, q_values):
    return np.sqrt(np.array(i_values)**2 + np.array(q_values)**2)

def plot_signals(amplitude, sample_rate):
    # Create a figure with two subplots
    fig, axs = plt.subplots(2, 1, figsize=(12, 8))

    # Plot Amplitude
    axs[0].plot(amplitude)
    axs[0].set_title("Amplitude of the Signal")
    axs[0].set_xlabel("Sample Number")
    axs[0].set_ylabel("Amplitude")
    axs[0].grid(True)

    # Compute the FFT
    fft_values = np.fft.fft(amplitude)
    fft_values[0] = 0  # Remove DC component
    freqs = np.fft.fftfreq(len(amplitude), 1 / sample_rate)

    # Only plot the positive frequencies
    mask = freqs > 0
    axs[1].plot(freqs[mask], np.abs(fft_values)[mask])
    axs[1].set_title("FFT of the Amplitude")
    axs[1].set_xlabel("Frequency (Hz)")
    axs[1].set_ylabel("Magnitude")
    axs[1].grid(True)

    # Adjust layout to prevent overlap
    plt.tight_layout()
    plt.show()


def plot_fft_for_all_files(directory, sample_rate=20e6, center_freq = 2.412e9):
    files = [f for f in os.listdir(directory) if f.endswith('.txt')]
    num_files = len(files)
    num_plots_per_figure = 4  # number of subplots per figure

    # Process each file and plot in batches of 4
    for i in range(0, num_files, num_plots_per_figure):
        batch_files = files[i:i + num_plots_per_figure]
        fig, axs = plt.subplots(len(batch_files), 1, figsize=(12, 3 * len(batch_files)), squeeze=False)

        for j, filename in enumerate(batch_files):
            file_path = os.path.join(directory, filename)
            i_values, q_values = parse_hex_file(file_path)
            
            complex_list = [complex(i, q) for i, q in zip(i_values, q_values)]
            PSD = np.abs(np.fft.fft(complex_list))**2 / (len(complex_list)*sample_rate)
            PSD_log = 10.0*np.log10(PSD)
            PSD_shifted = np.fft.fftshift(PSD_log)
            
            f = np.arange(sample_rate/-2.0, sample_rate/2.0, sample_rate/len(complex_list))
            f += center_freq

            axs[j, 0].plot(f, PSD_shifted)
            axs[j, 0].set_title(filename)
            axs[j, 0].set_xlabel("Frequency (Hz)")
            axs[j, 0].set_ylabel("Magnitude")
            axs[j, 0].grid(True)

        plt.subplots_adjust(left=0.08, bottom=0.049,right=0.988,top=0.969,wspace=0.2,hspace=0.665)  # Adjust the vertical spacing between subplots
        #plt.tight_layout()
        plt.show()


# Usage example
directory_path = '.'  # Replace with your directory path
plot_fft_for_all_files(directory_path)